# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'edit_form.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_EditFormWindow(object):
    def setupUi(self, EditFormWindow):
        if not EditFormWindow.objectName():
            EditFormWindow.setObjectName(u"EditFormWindow")
        EditFormWindow.resize(1090, 877)
        self.actionLoad_JSON = QAction(EditFormWindow)
        self.actionLoad_JSON.setObjectName(u"actionLoad_JSON")
        self.actionSave_JSON = QAction(EditFormWindow)
        self.actionSave_JSON.setObjectName(u"actionSave_JSON")
        self.actionUpload_to_MR_HUB = QAction(EditFormWindow)
        self.actionUpload_to_MR_HUB.setObjectName(u"actionUpload_to_MR_HUB")
        self.centralwidget = QWidget(EditFormWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout_5 = QVBoxLayout(self.centralwidget)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.single_fields_widget = QWidget(self.centralwidget)
        self.single_fields_widget.setObjectName(u"single_fields_widget")
        self.formLayout = QFormLayout(self.single_fields_widget)
        self.formLayout.setObjectName(u"formLayout")
        self.label = QLabel(self.single_fields_widget)
        self.label.setObjectName(u"label")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.label)

        self.name_text = QLineEdit(self.single_fields_widget)
        self.name_text.setObjectName(u"name_text")

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.name_text)

        self.label_2 = QLabel(self.single_fields_widget)
        self.label_2.setObjectName(u"label_2")

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.label_2)

        self.label_3 = QLabel(self.single_fields_widget)
        self.label_3.setObjectName(u"label_3")

        self.formLayout.setWidget(3, QFormLayout.LabelRole, self.label_3)

        self.label_4 = QLabel(self.single_fields_widget)
        self.label_4.setObjectName(u"label_4")

        self.formLayout.setWidget(4, QFormLayout.LabelRole, self.label_4)

        self.label_5 = QLabel(self.single_fields_widget)
        self.label_5.setObjectName(u"label_5")

        self.formLayout.setWidget(5, QFormLayout.LabelRole, self.label_5)

        self.label_6 = QLabel(self.single_fields_widget)
        self.label_6.setObjectName(u"label_6")

        self.formLayout.setWidget(6, QFormLayout.LabelRole, self.label_6)

        self.label_7 = QLabel(self.single_fields_widget)
        self.label_7.setObjectName(u"label_7")

        self.formLayout.setWidget(7, QFormLayout.LabelRole, self.label_7)

        self.label_8 = QLabel(self.single_fields_widget)
        self.label_8.setObjectName(u"label_8")

        self.formLayout.setWidget(8, QFormLayout.LabelRole, self.label_8)

        self.longDescription_label = QLabel(self.single_fields_widget)
        self.longDescription_label.setObjectName(u"longDescription_label")

        self.formLayout.setWidget(9, QFormLayout.LabelRole, self.longDescription_label)

        self.label_10 = QLabel(self.single_fields_widget)
        self.label_10.setObjectName(u"label_10")

        self.formLayout.setWidget(10, QFormLayout.LabelRole, self.label_10)

        self.label_11 = QLabel(self.single_fields_widget)
        self.label_11.setObjectName(u"label_11")

        self.formLayout.setWidget(11, QFormLayout.LabelRole, self.label_11)

        self.label_12 = QLabel(self.single_fields_widget)
        self.label_12.setObjectName(u"label_12")

        self.formLayout.setWidget(12, QFormLayout.LabelRole, self.label_12)

        self.imageFilename_text = QLineEdit(self.single_fields_widget)
        self.imageFilename_text.setObjectName(u"imageFilename_text")

        self.formLayout.setWidget(4, QFormLayout.FieldRole, self.imageFilename_text)

        self.shortDescription_text = QLineEdit(self.single_fields_widget)
        self.shortDescription_text.setObjectName(u"shortDescription_text")

        self.formLayout.setWidget(3, QFormLayout.FieldRole, self.shortDescription_text)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.imagePath_text = QLineEdit(self.single_fields_widget)
        self.imagePath_text.setObjectName(u"imagePath_text")
        self.imagePath_text.setEnabled(False)

        self.horizontalLayout.addWidget(self.imagePath_text)

        self.imageBrowse_button = QPushButton(self.single_fields_widget)
        self.imageBrowse_button.setObjectName(u"imageBrowse_button")

        self.horizontalLayout.addWidget(self.imageBrowse_button)


        self.formLayout.setLayout(5, QFormLayout.FieldRole, self.horizontalLayout)

        self.mainURL_text = QLineEdit(self.single_fields_widget)
        self.mainURL_text.setObjectName(u"mainURL_text")

        self.formLayout.setWidget(6, QFormLayout.FieldRole, self.mainURL_text)

        self.repoURL_text = QLineEdit(self.single_fields_widget)
        self.repoURL_text.setObjectName(u"repoURL_text")

        self.formLayout.setWidget(7, QFormLayout.FieldRole, self.repoURL_text)

        self.principalDevelopers_text = QLineEdit(self.single_fields_widget)
        self.principalDevelopers_text.setObjectName(u"principalDevelopers_text")

        self.formLayout.setWidget(8, QFormLayout.FieldRole, self.principalDevelopers_text)

        self.longDescription_text = QPlainTextEdit(self.single_fields_widget)
        self.longDescription_text.setObjectName(u"longDescription_text")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.longDescription_text.sizePolicy().hasHeightForWidth())
        self.longDescription_text.setSizePolicy(sizePolicy)

        self.formLayout.setWidget(9, QFormLayout.FieldRole, self.longDescription_text)

        self.dateAdded_text = QLineEdit(self.single_fields_widget)
        self.dateAdded_text.setObjectName(u"dateAdded_text")

        self.formLayout.setWidget(10, QFormLayout.FieldRole, self.dateAdded_text)

        self.dateUpdated_text = QLineEdit(self.single_fields_widget)
        self.dateUpdated_text.setObjectName(u"dateUpdated_text")

        self.formLayout.setWidget(11, QFormLayout.FieldRole, self.dateUpdated_text)

        self.category_box = QComboBox(self.single_fields_widget)
        self.category_box.addItem("")
        self.category_box.addItem("")
        self.category_box.addItem("")
        self.category_box.addItem("")
        self.category_box.addItem("")
        self.category_box.addItem("")
        self.category_box.addItem("")
        self.category_box.addItem("")
        self.category_box.addItem("")
        self.category_box.setObjectName(u"category_box")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.category_box.sizePolicy().hasHeightForWidth())
        self.category_box.setSizePolicy(sizePolicy1)

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.category_box)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.citationString_text = QLineEdit(self.single_fields_widget)
        self.citationString_text.setObjectName(u"citationString_text")
        sizePolicy2 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy2.setHorizontalStretch(1)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.citationString_text.sizePolicy().hasHeightForWidth())
        self.citationString_text.setSizePolicy(sizePolicy2)

        self.horizontalLayout_6.addWidget(self.citationString_text)

        self.citationTest_button = QPushButton(self.single_fields_widget)
        self.citationTest_button.setObjectName(u"citationTest_button")
        sizePolicy3 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.citationTest_button.sizePolicy().hasHeightForWidth())
        self.citationTest_button.setSizePolicy(sizePolicy3)
        self.citationTest_button.setBaseSize(QSize(1, 0))

        self.horizontalLayout_6.addWidget(self.citationTest_button)


        self.formLayout.setLayout(12, QFormLayout.FieldRole, self.horizontalLayout_6)


        self.horizontalLayout_3.addWidget(self.single_fields_widget)

        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout = QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.keywords_table = QTableView(self.groupBox)
        self.keywords_table.setObjectName(u"keywords_table")

        self.verticalLayout.addWidget(self.keywords_table)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.addKeyword_button = QPushButton(self.groupBox)
        self.addKeyword_button.setObjectName(u"addKeyword_button")

        self.horizontalLayout_2.addWidget(self.addKeyword_button)

        self.removeKeyword_button = QPushButton(self.groupBox)
        self.removeKeyword_button.setObjectName(u"removeKeyword_button")

        self.horizontalLayout_2.addWidget(self.removeKeyword_button)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.groupBox_2 = QGroupBox(self.groupBox)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.verticalLayout_3 = QVBoxLayout(self.groupBox_2)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.extraResources_Table = QTableView(self.groupBox_2)
        self.extraResources_Table.setObjectName(u"extraResources_Table")

        self.verticalLayout_3.addWidget(self.extraResources_Table)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.addExtraResources_button = QPushButton(self.groupBox_2)
        self.addExtraResources_button.setObjectName(u"addExtraResources_button")

        self.horizontalLayout_4.addWidget(self.addExtraResources_button)

        self.removeExtraResources_button = QPushButton(self.groupBox_2)
        self.removeExtraResources_button.setObjectName(u"removeExtraResources_button")

        self.horizontalLayout_4.addWidget(self.removeExtraResources_button)


        self.verticalLayout_3.addLayout(self.horizontalLayout_4)


        self.verticalLayout.addWidget(self.groupBox_2)


        self.verticalLayout_2.addWidget(self.groupBox)


        self.horizontalLayout_3.addLayout(self.verticalLayout_2)


        self.verticalLayout_5.addLayout(self.horizontalLayout_3)

        self.groupBox_3 = QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.horizontalLayout_5 = QHBoxLayout(self.groupBox_3)
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.reference_table = QTableView(self.groupBox_3)
        self.reference_table.setObjectName(u"reference_table")

        self.horizontalLayout_5.addWidget(self.reference_table)

        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.addReference_button = QPushButton(self.groupBox_3)
        self.addReference_button.setObjectName(u"addReference_button")

        self.verticalLayout_4.addWidget(self.addReference_button)

        self.removeReference_button = QPushButton(self.groupBox_3)
        self.removeReference_button.setObjectName(u"removeReference_button")

        self.verticalLayout_4.addWidget(self.removeReference_button)


        self.horizontalLayout_5.addLayout(self.verticalLayout_4)


        self.verticalLayout_5.addWidget(self.groupBox_3)

        EditFormWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(EditFormWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 1090, 32))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        EditFormWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(EditFormWindow)
        self.statusbar.setObjectName(u"statusbar")
        EditFormWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menuFile.addAction(self.actionLoad_JSON)
        self.menuFile.addAction(self.actionSave_JSON)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionUpload_to_MR_HUB)

        self.retranslateUi(EditFormWindow)

        QMetaObject.connectSlotsByName(EditFormWindow)
    # setupUi

    def retranslateUi(self, EditFormWindow):
        EditFormWindow.setWindowTitle(QCoreApplication.translate("EditFormWindow", u"MainWindow", None))
        self.actionLoad_JSON.setText(QCoreApplication.translate("EditFormWindow", u"Load JSON...", None))
        self.actionSave_JSON.setText(QCoreApplication.translate("EditFormWindow", u"Save JSON...", None))
        self.actionUpload_to_MR_HUB.setText(QCoreApplication.translate("EditFormWindow", u"Prepare MR-Hub submission...", None))
        self.label.setText(QCoreApplication.translate("EditFormWindow", u"Name:", None))
        self.name_text.setPlaceholderText(QCoreApplication.translate("EditFormWindow", u"Package name", None))
        self.label_2.setText(QCoreApplication.translate("EditFormWindow", u"Category:", None))
        self.label_3.setText(QCoreApplication.translate("EditFormWindow", u"Short Description:", None))
        self.label_4.setText(QCoreApplication.translate("EditFormWindow", u"Image Filename (remote):", None))
        self.label_5.setText(QCoreApplication.translate("EditFormWindow", u"Select image:", None))
        self.label_6.setText(QCoreApplication.translate("EditFormWindow", u"Main URL:", None))
        self.label_7.setText(QCoreApplication.translate("EditFormWindow", u"Repo URL:", None))
        self.label_8.setText(QCoreApplication.translate("EditFormWindow", u"Principal Developers:", None))
        self.longDescription_label.setText(QCoreApplication.translate("EditFormWindow", u"<html><head/><body><p>Long description:</p><p>({:d}/200 words)</p></body></html>", None))
        self.label_10.setText(QCoreApplication.translate("EditFormWindow", u"Date added to MR Hub:", None))
        self.label_11.setText(QCoreApplication.translate("EditFormWindow", u"Date sofware last updated:", None))
        self.label_12.setText(QCoreApplication.translate("EditFormWindow", u"Citation search string:", None))
        self.imageFilename_text.setPlaceholderText(QCoreApplication.translate("EditFormWindow", u"your_image.png", None))
        self.shortDescription_text.setPlaceholderText(QCoreApplication.translate("EditFormWindow", u"One-line description", None))
        self.imageBrowse_button.setText(QCoreApplication.translate("EditFormWindow", u"Browse...", None))
        self.mainURL_text.setPlaceholderText(QCoreApplication.translate("EditFormWindow", u"https://www.your.website/", None))
        self.repoURL_text.setPlaceholderText(QCoreApplication.translate("EditFormWindow", u"https://github.com/developer/repo", None))
        self.dateAdded_text.setInputMask(QCoreApplication.translate("EditFormWindow", u"9999-99-99", None))
        self.dateAdded_text.setPlaceholderText(QCoreApplication.translate("EditFormWindow", u"2020-01-01", None))
        self.dateUpdated_text.setInputMask(QCoreApplication.translate("EditFormWindow", u"9999-99-99", None))
        self.dateUpdated_text.setPlaceholderText(QCoreApplication.translate("EditFormWindow", u"YYYY-MM-DD", None))
        self.category_box.setItemText(0, QCoreApplication.translate("EditFormWindow", u"Reconstruction", None))
        self.category_box.setItemText(1, QCoreApplication.translate("EditFormWindow", u"Data format", None))
        self.category_box.setItemText(2, QCoreApplication.translate("EditFormWindow", u"Pulse sequences", None))
        self.category_box.setItemText(3, QCoreApplication.translate("EditFormWindow", u"Educational", None))
        self.category_box.setItemText(4, QCoreApplication.translate("EditFormWindow", u"Simulation", None))
        self.category_box.setItemText(5, QCoreApplication.translate("EditFormWindow", u"Visualisation", None))
        self.category_box.setItemText(6, QCoreApplication.translate("EditFormWindow", u"Spectroscopy", None))
        self.category_box.setItemText(7, QCoreApplication.translate("EditFormWindow", u"Image processing", None))
        self.category_box.setItemText(8, QCoreApplication.translate("EditFormWindow", u"Multipurpose", None))

        self.citationString_text.setPlaceholderText(QCoreApplication.translate("EditFormWindow", u"DOI or paperID for Semantic Scholar", None))
        self.citationTest_button.setText(QCoreApplication.translate("EditFormWindow", u"Test", None))
        self.groupBox.setTitle(QCoreApplication.translate("EditFormWindow", u"Keywords", None))
        self.addKeyword_button.setText(QCoreApplication.translate("EditFormWindow", u"Add", None))
        self.removeKeyword_button.setText(QCoreApplication.translate("EditFormWindow", u"Remove", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("EditFormWindow", u"Extra Resources", None))
        self.addExtraResources_button.setText(QCoreApplication.translate("EditFormWindow", u"Add", None))
        self.removeExtraResources_button.setText(QCoreApplication.translate("EditFormWindow", u"Remove", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("EditFormWindow", u"References", None))
        self.addReference_button.setText(QCoreApplication.translate("EditFormWindow", u"Add", None))
        self.removeReference_button.setText(QCoreApplication.translate("EditFormWindow", u"Remove", None))
        self.menuFile.setTitle(QCoreApplication.translate("EditFormWindow", u"File...", None))
    # retranslateUi




import os
import sys
import webbrowser

from PySide2.QtGui import QRegExpValidator

from .edit_form_ui import Ui_EditFormWindow
from .github_credentials_ui import Ui_GithubCredentialsDialog
from PySide2.QtWidgets import QMainWindow, QHeaderView, QFileDialog, QMessageBox, QDialog, QApplication
from PySide2.QtCore import QAbstractTableModel, Qt, Slot, QRegExp
import json
import requests
from .GitTools import GitTools, GitError, DEFAULT_LOCAL_DIR
from datetime import datetime


class GithubCredentialsDialog(QDialog, Ui_GithubCredentialsDialog):
    def __init__(self, parent):
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.userpass_widget.setVisible(False)
        self.buttonBox.accepted.connect(lambda: self.exit_dialog(True))
        self.buttonBox.rejected.connect(lambda: self.exit_dialog(False))
        self.localFolder_Browse_Button.clicked.connect(self.browse_folder)

        branchValidator = QRegExpValidator(QRegExp(r'[A-Za-z0-9_-]+'))
        self.newBranch_text.setValidator(branchValidator)

        self.credentials = None
        self.accepted = False
        self.local_folder = None
        self.branch = None

        self.setModal(True)

    def set_folder(self, folder):
        self.localFolder_text.setText(folder)

    def browse_folder(self):
        d = QFileDialog.getExistingDirectory(self, 'Select the folder for the local repository')
        self.localFolder_text.setText(d)

    def exit_dialog(self, accepted):
        self.accepted = accepted
        self.close()

    def closeEvent(self, event):
        if self.userpass_radio.isChecked():
            self.credentials = (self.username_text.text(), self.password_text.text())
        else:
            self.credentials = self.accessToken_text.text()
        self.local_folder = self.localFolder_text.text()
        self.branch = self.newBranch_text.text()


class TableModel(QAbstractTableModel):
    def __init__(self, header):
        QAbstractTableModel.__init__(self)
        self.ncols = len(header)
        self.header = header
        self.keywordList = []

    def headerData(self, section, orientation, role):
        if role == Qt.DisplayRole:
            if orientation == Qt.Horizontal:
                return self.header[section]
            else:
                return str(section+1)

    def data(self, index, role):
        if role == Qt.DisplayRole:
            return self.keywordList[index.row()][self.header[index.column()]]

    def rowCount(self, index):
        return len(self.keywordList)

    def columnCount(self, index):
        return self.ncols

    def setData(self, index, value, role=None):
        if role == Qt.EditRole:
            self.keywordList[index.row()][self.header[index.column()]] = value
            return True

    def flags(self, index):
        return Qt.ItemIsEditable | QAbstractTableModel.flags(self, index)

    def insertRow(self, row, parent=None, *args, **kwargs):
        self.keywordList.insert(row, {k:'' for k in self.header})
        print("inserting row. Len", len(self.keywordList))
        self.layoutChanged.emit()
        return True

    def appendRow(self):
        return self.insertRow(len(self.keywordList)+1)

    def deleteRow(self, row):
        del self.keywordList[row]
        self.layoutChanged.emit()

    def getList(self):
        return self.keywordList

    def setList(self, dataList):
        self.keywordList = dataList
        self.layoutChanged.emit()

def table_row_delete(model, view):
    try:
        row = view.selectedIndexes()[0].row()
    except:
        print('No row selected')
        return
    model.deleteRow(row)
    

class EditForm(QMainWindow, Ui_EditFormWindow):

    def __init__(self):
        QMainWindow.__init__(self)
        self.setupUi(self)
        self.setWindowTitle('MR-HUB GUI')

        self.keyword_model = TableModel(['Keyword'])
        self.keywords_table.setModel(self.keyword_model)
        self.keywords_table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)

        self.addKeyword_button.clicked.connect(self.keyword_model.appendRow)
        self.removeKeyword_button.clicked.connect(lambda : table_row_delete(self.keyword_model, self.keywords_table))

        self.reference_model = TableModel(['Description', 'URL'])
        self.reference_table.setModel(self.reference_model)
        self.reference_table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        self.addReference_button.clicked.connect(self.reference_model.appendRow)
        self.removeReference_button.clicked.connect(lambda : table_row_delete(self.reference_model, self.reference_table))

        self.extraResources_model = TableModel(['Resource type', 'URL'])
        self.extraResources_Table.setModel(self.extraResources_model)
        self.extraResources_Table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        self.addExtraResources_button.clicked.connect(self.extraResources_model.appendRow)
        self.removeExtraResources_button.clicked.connect(lambda: table_row_delete(self.extraResources_model, self.extraResources_Table))

        self.dateAdded_text.setText(datetime.now().strftime('%Y-%m-%d'))
        self.dateUpdated_text.setText(datetime.now().strftime('%Y-%m-%d'))
        self.longDescription_template = self.longDescription_label.text()

        self.longDescription_text.textChanged.connect(self.update_longDescription_label)
        self.update_longDescription_label()

        self.imageBrowse_button.clicked.connect(self.set_image_path)

        self.actionLoad_JSON.triggered.connect(self.loadJSON)
        self.actionSave_JSON.triggered.connect(self.saveJSON)

        self.actionUpload_to_MR_HUB.triggered.connect(self.prepare_mr_hub)

        self.citationTest_button.clicked.connect(self.test_semantic_scholar)

    def update_longDescription_label(self):
        n_words = len(self.longDescription_text.toPlainText().split())
        self.longDescription_label.setText(self.longDescription_template.format(n_words))

    def get_output_dict(self):
        def make_keyword_list():
            return [v['Keyword'] for v in self.keyword_model.getList()]

        def make_reference_list():
            return [{'referenceText': v['Description'], 'referenceURL': v['URL']} for v in
                    self.reference_model.getList()]

        def make_resource_list():
            return [{'resourceType': v['Resource type'], 'URL': v['URL']} for v in
                    self.extraResources_model.getList()]

        output = {
            'name': self.name_text.text(),
            'category': self.category_box.currentText(),
            'shortDescription': self.shortDescription_text.text(),
            'imageFile': self.imageFilename_text.text(),
            'mainURL': self.mainURL_text.text(),
            'repoURL': self.repoURL_text.text(),
            'principalDevelopers': self.principalDevelopers_text.text(),
            'longDescription': self.longDescription_text.toPlainText(),
            'keywords': make_keyword_list(),
            'keyReferences': make_reference_list(),
            'extraResources': make_resource_list(),
            'dateAddedToMRHub': self.dateAdded_text.text(),
            'dateSoftwareLastUpdated': self.dateUpdated_text.text(),
            'citationSearchString': self.citationString_text.text(),
            'citationCount': ''
        }

        return output

    def load_dict(self, input):
        def load_keyword_list(keywords):
            self.keyword_model.setList(
                [{'Keyword': k} for k in keywords]
            )

        def load_reference_list(references):
            self.reference_model.setList(
                [{'Description': v['referenceText'],
                  'URL': v['referenceURL']} for v in references]
            )

        def load_resource_list(resources):
            self.extraResources_model.setList(
                [{'Resource type': v['resourceType'],
                  'URL': v['URL']} for v in resources]
            )

        self.name_text.setText(input['name'])
        self.category_box.setCurrentText(input['category'])
        self.shortDescription_text.setText(input['shortDescription'])
        self.imageFilename_text.setText(input['imageFile'])
        self.mainURL_text.setText(input['mainURL'])
        self.repoURL_text.setText(input['repoURL'])
        self.principalDevelopers_text.setText(input['principalDevelopers'])
        self.longDescription_text.setPlainText(input['longDescription'])
        load_keyword_list(input['keywords'])
        load_reference_list(input['keyReferences'])
        load_resource_list(input['extraResources'])
        self.dateAdded_text.setText(input['dateAddedToMRHub'])
        self.dateUpdated_text.setText(input['dateSoftwareLastUpdated'])
        self.citationString_text.setText(input['citationSearchString'])

    @Slot()
    def loadJSON(self):
        jsonFile, _ = QFileDialog.getOpenFileName(self, caption='Import JSON file',
                                                   filter='JSON files (*.json);;All files (*.*)')

        if not jsonFile: return
        inputDict = json.load(open(jsonFile, 'r'))
        try:
            self.load_dict(inputDict)
        except:
            QMessageBox.critical(self, 'JSON error', 'Malformed JSON File')

    @Slot()
    def saveJSON(self):
        jsonFile, _ = QFileDialog.getSaveFileName(self, caption='Save JSON file',
                                                  filter='JSON files (*.json);;All files (*.*)')

        if not jsonFile: return
        json.dump(self.get_output_dict(), open(jsonFile, 'w'), indent=2)

    @Slot()
    def test_semantic_scholar(self):
        base_url = 'https://api.semanticscholar.org/v1/paper/'
        ss_url = base_url + self.citationString_text.text()
        r = requests.get(ss_url)
        if r.status_code == 404:
            QMessageBox.critical(self, 'SemanticScholar error', 'Paper not found in Semantic Scholar')
        elif r.status_code == 200:
            QMessageBox.information(self, 'Paper found', 'The following paper was found:\n\n' + r.json()['title'])
        else:
            QMessageBox.critical(self, 'SemanticScholar error', 'Unspecified error with Semantic Scholar')

    @Slot()
    def set_image_path(self):
        image_path, _ = QFileDialog.getOpenFileName(self, caption='Select an image file',
                                                    filter='Image files (*.jpg *.png *.gif *.svg);;All files (*.*)')
        self.imagePath_text.setText(image_path)
        if not self.imageFilename_text.text():
            self.imageFilename_text.setText(os.path.basename(image_path))

    @Slot()
    def prepare_mr_hub(self):

        output_dict = self.get_output_dict()

        if output_dict['imageFile'] and not self.imagePath_text.text():
            ans = QMessageBox.warning(self, 'Image missing', 'An image name is specified but no image file is selected! Continue anyway?',
                                      QMessageBox.Ok, QMessageBox.Cancel)
            if ans == QMessageBox.Cancel:
                return

        _, imageFile_ext = os.path.splitext(output_dict['imageFile'])
        _, imagePath_ext = os.path.splitext(self.imagePath_text.text())
        if imagePath_ext and imageFile_ext != imagePath_ext:
            ans = QMessageBox.warning(self, 'Image format',
                                      'The image name and the image to upload have different extensions! Continue anyway?',
                                      QMessageBox.Ok, QMessageBox.Cancel)
            if ans == QMessageBox.Cancel:
                return


        ans = QMessageBox.information(self, 'Forking the repo',
"""This will now create a Git repository that needs to be pushed to Github and from which a pull request can be created.

You need the Git executable to be present in the path and properly configured with a user name and email.
You also need a Github account.

The program will now:
1) fork the original mr-hub repository into your own Github account;
2) clone the repository locally;
3) create a new branch;
4) perform the required changes;
5) push the new branch to Github;
6) open the browser to perform the pull request.

You will need check the changes and confirm the pull request. 
""", QMessageBox.Ok, QMessageBox.Cancel)

        if ans == QMessageBox.Cancel:
            return

        if not GitTools.check_git_settings():
            QMessageBox.critical(self, 'Git error', 'Git Settings error. Either git is unavailable, or the username is not set.\n' +
                                                    'Install git and/or run the following commands:\n' +
                                                    'git config --global user.name <your username>\n' +
                                                    'git config --global user.email <your email>')
            return

        credential_dialog = GithubCredentialsDialog(self)
        credential_dialog.set_folder(os.path.abspath(DEFAULT_LOCAL_DIR))
        credential_dialog.exec_()
        if not credential_dialog.accepted:
            return

        credentials = credential_dialog.credentials

        if (isinstance(credentials, str) and not credentials) or (
                isinstance(credentials, tuple) and (not credentials[0] or not credentials[1])):
            QMessageBox.critical(self, 'Git error', 'Credentials cannot be empty')
            return


        new_branch = credential_dialog.branch
        if not new_branch:
            QMessageBox.critical(self, 'Git error', 'Specify a new branch name')
            return


        local_folder = credential_dialog.local_folder
        if not local_folder:
            QMessageBox.critical(self, 'Git error', 'Specify a folder to store the repository')
            return

        git_tools = GitTools(credentials, new_branch, local_folder)
        # initialize the github repo
        try:
            git_tools.initialize_github()
        except GitError as err:
            QMessageBox.critical(self, 'Git Error', 'Error while initializing the repository: ' + str(err))
            return

        # if an image is specified, add it
        if self.imagePath_text.text() and output_dict['imageFile']:
            git_tools.copy_image(self.imagePath_text.text(), output_dict['imageFile'])

        try:
            git_tools.add_package(output_dict)
        except GitError as err:
            QMessageBox.critical(self, 'Git Error', 'Error while modifying the repository: ' + str(err))
            return

        webbrowser.open(git_tools.github_compare_url())

        ans = QMessageBox.information(self, 'Delete the local repo',
                                      f'If all went well, you can now delete the local repository.\nWould you like to delete {git_tools.local_dir}?'
                                      , QMessageBox.Yes, QMessageBox.No)

        if ans == QMessageBox.Yes:
            git_tools.delete_local_repository()


def main():
    app = QApplication(sys.argv)
    win = EditForm()
    win.show()
    sys.exit(app.exec_())



from github import Github
from github.GithubException import BadCredentialsException
import subprocess
import os
import shutil
import json

BASE_REPO_NAME = 'ismrm/mrhub'
MAIN_BRANCH = 'master'

DEFAULT_LOCAL_DIR = 'local_mrhub_repo'

settings = {
    'LOCAL_DIR': 'local_mrhub_repo',
    'USER': '__token__',
    'PASS': '',
    'FORKED_REPO': None
}


class GitError(Exception):
    pass

class GitTools:

    def __init__(self, credentials, new_branch_name, local_dir = DEFAULT_LOCAL_DIR):
        if isinstance(credentials, tuple):
            self.github = Github(*credentials)
            self.user = credentials[0]
            self.password = credentials[1]
        else:
            self.github = Github(credentials)
            self.user = '__token__'
            self.password = credentials

        self.local_dir = os.path.abspath(local_dir)
        self.new_branch_name = new_branch_name
        self.forked_repo = None

    def initialize_github(self):
        # get authenticated user
        github_user = self.github.get_user()

        try:
            print(github_user.name)
        except BadCredentialsException:
            raise GitError('Authentication failed')

        # get a reference to the base repo
        base_repo = self.github.get_repo(BASE_REPO_NAME)

        # make a fork
        self.forked_repo = github_user.create_fork(base_repo)

        return_code, out, err = self.git(f'clone {self.github_repo_url()} "{self.local_dir}"')
        if return_code:
            if 'already exists' in err:
                print('Warning: repo already exists')
            else:
                raise GitError('Error during cloning')

        self.git(f'checkout {MAIN_BRANCH}', self.local_dir)
        self.git(f'remote add upstream {self.github_repo_url(base_repo)}', self.local_dir)
        self.git('pull', self.local_dir)
        self.git('fetch upstream', self.local_dir)
        self.git(f'merge -s recursive -Xtheirs upstream/{MAIN_BRANCH}', self.local_dir)
        ret, out, err = self.git(f'checkout -b "{self.new_branch_name}"', self.local_dir)
        if 'fatal' in err.lower():
            raise GitError('Branch already exists')

    def github_repo_url(self, repo=None):
        if repo is None:
            repo = self.forked_repo
        return f'https://github.com/{repo.full_name}.git'

    def github_compare_url(self):
        if self.forked_repo is None:
            raise GitError('Repository not forked')
        return f"https://github.com/{self.forked_repo.full_name}/compare/{self.get_branch_name()}"

    def create_askpass_posix(self):
        askpass_file = os.path.join(self.local_dir, 'askpass.sh')
        with open(askpass_file, 'w') as f:
            f.write(
f"""#!/bin/sh
if echo $1 | grep -q Username
then
    echo {self.user}
fi
if echo $1 | grep -q Password
then
    echo {self.password}
fi  
""")
        os.chmod(askpass_file, 0o777)
        return os.path.abspath(askpass_file)

    def create_askpass_win(self):
        askpass_file = os.path.join(self.local_dir, 'askpass.bat')
        with open(askpass_file, 'w') as f:
            f.write(
f"""@set arg=%%~1
@if (%%arg:~0,8%%)==(Username) echo {self.user}
@if (%%arg:~0,8%%)==(Password) echo {self.password}
""")
        return os.path.abspath(askpass_file)

    def create_askpass_file(self):
        if os.name == 'posix':
            return self.create_askpass_posix()
        elif os.name == 'nt':
            return self.create_askpass_win()
        else:
            raise GitError('OS not supported')

    @staticmethod
    def git_command(git_options, working_dir=None):
        cmd = 'git '
        if working_dir:
            cmd += f'-C "{working_dir}" '

        print(cmd + git_options)
        proc = subprocess.run(cmd + git_options, capture_output=True, shell=True)
        print('Stdout')
        print(proc.stdout)
        print('Stderr')
        print(proc.stderr)

        return proc.returncode, proc.stdout.decode(), proc.stderr.decode()

    def git(self, git_options, working_dir=None, send_auth=False):
        if send_auth:
            askpass_file = self.create_askpass_file()
            os.environ['GIT_ASKPASS'] = askpass_file

        ret, out, err = GitTools.git_command(git_options, working_dir)

        if send_auth:
            os.remove(askpass_file)
            del os.environ['GIT_ASKPASS']

        return ret, out, err

    @staticmethod
    def check_git_settings():
        ret, out, err = GitTools.git_command('config --get user.name')
        if ret or not out: return False
        ret, out, err = GitTools.git_command('config --get user.email')
        if ret or not out: return False
        return True

    def copy_image(self, image_file, image_name):
        repo_dir = os.path.abspath(self.local_dir)
        shutil.copy(image_file, os.path.join(repo_dir, 'images_packages', image_name))
        self.git(f'add {os.path.join("images_packages", image_name)}', self.local_dir)

    def add_package(self, package_dict):
        original_dir = os.getcwd()
        os.chdir(self.local_dir)
        current_package_lines = open(os.path.join('_data', 'projects.json'), 'r', encoding='utf-8').readlines()
        # find opening [
        bracket_line = -1
        for number, line in enumerate(current_package_lines):
            if line.strip() == '[':
                bracket_line = number
                break

        assert bracket_line > -1, "Malformed project.json file"
        package_json = json.dumps(package_dict, indent=2) + ','
        # insert lines in inverse order
        for line in package_json.splitlines()[::-1]:
            current_package_lines.insert(bracket_line + 1, ' ' * 2 + line + '\n')

        with open(os.path.join('_data', 'projects.json'), 'w', encoding='utf-8') as f:
            f.writelines(current_package_lines)

        self.git('commit -a -m "Package addition - generated by mr-hub-gui"', working_dir=self.local_dir)

        self.git(f'push -u origin {self.get_branch_name()}', working_dir=self.local_dir, send_auth=True)

        os.chdir(original_dir)

    def get_branch_name(self):
        ret, out, err = self.git('status', self.local_dir)
        for line in out.splitlines():
            if line.lower().startswith('on branch '):
                return line[len('on branch '):]
        return None

    def delete_local_repository(self):
        shutil.rmtree(self.local_dir)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'github_credentials.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_GithubCredentialsDialog(object):
    def setupUi(self, GithubCredentialsDialog):
        if not GithubCredentialsDialog.objectName():
            GithubCredentialsDialog.setObjectName(u"GithubCredentialsDialog")
        GithubCredentialsDialog.resize(476, 324)
        self.verticalLayout = QVBoxLayout(GithubCredentialsDialog)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.groupBox = QGroupBox(GithubCredentialsDialog)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout_2 = QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.accessToken_radio = QRadioButton(self.groupBox)
        self.accessToken_radio.setObjectName(u"accessToken_radio")
        self.accessToken_radio.setChecked(True)

        self.horizontalLayout_3.addWidget(self.accessToken_radio)

        self.userpass_radio = QRadioButton(self.groupBox)
        self.userpass_radio.setObjectName(u"userpass_radio")

        self.horizontalLayout_3.addWidget(self.userpass_radio)


        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.accessToken_widget = QWidget(self.groupBox)
        self.accessToken_widget.setObjectName(u"accessToken_widget")
        self.horizontalLayout_2 = QHBoxLayout(self.accessToken_widget)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.label_3 = QLabel(self.accessToken_widget)
        self.label_3.setObjectName(u"label_3")

        self.horizontalLayout_2.addWidget(self.label_3)

        self.accessToken_text = QLineEdit(self.accessToken_widget)
        self.accessToken_text.setObjectName(u"accessToken_text")

        self.horizontalLayout_2.addWidget(self.accessToken_text)


        self.verticalLayout_2.addWidget(self.accessToken_widget)

        self.userpass_widget = QWidget(self.groupBox)
        self.userpass_widget.setObjectName(u"userpass_widget")
        self.horizontalLayout = QHBoxLayout(self.userpass_widget)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.label = QLabel(self.userpass_widget)
        self.label.setObjectName(u"label")

        self.horizontalLayout.addWidget(self.label)

        self.username_text = QLineEdit(self.userpass_widget)
        self.username_text.setObjectName(u"username_text")

        self.horizontalLayout.addWidget(self.username_text)

        self.label_2 = QLabel(self.userpass_widget)
        self.label_2.setObjectName(u"label_2")

        self.horizontalLayout.addWidget(self.label_2)

        self.password_text = QLineEdit(self.userpass_widget)
        self.password_text.setObjectName(u"password_text")

        self.horizontalLayout.addWidget(self.password_text)


        self.verticalLayout_2.addWidget(self.userpass_widget)


        self.verticalLayout.addWidget(self.groupBox)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.label_4 = QLabel(GithubCredentialsDialog)
        self.label_4.setObjectName(u"label_4")

        self.horizontalLayout_4.addWidget(self.label_4)

        self.newBranch_text = QLineEdit(GithubCredentialsDialog)
        self.newBranch_text.setObjectName(u"newBranch_text")

        self.horizontalLayout_4.addWidget(self.newBranch_text)


        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.label_5 = QLabel(GithubCredentialsDialog)
        self.label_5.setObjectName(u"label_5")

        self.horizontalLayout_5.addWidget(self.label_5)

        self.localFolder_text = QLineEdit(GithubCredentialsDialog)
        self.localFolder_text.setObjectName(u"localFolder_text")
        self.localFolder_text.setEnabled(False)

        self.horizontalLayout_5.addWidget(self.localFolder_text)

        self.localFolder_Browse_Button = QPushButton(GithubCredentialsDialog)
        self.localFolder_Browse_Button.setObjectName(u"localFolder_Browse_Button")

        self.horizontalLayout_5.addWidget(self.localFolder_Browse_Button)


        self.verticalLayout.addLayout(self.horizontalLayout_5)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        self.buttonBox = QDialogButtonBox(GithubCredentialsDialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)

        self.verticalLayout.addWidget(self.buttonBox)


        self.retranslateUi(GithubCredentialsDialog)
        self.buttonBox.accepted.connect(GithubCredentialsDialog.accept)
        self.buttonBox.rejected.connect(GithubCredentialsDialog.reject)
        self.accessToken_radio.toggled.connect(self.accessToken_widget.setVisible)
        self.userpass_radio.toggled.connect(self.userpass_widget.setVisible)

        QMetaObject.connectSlotsByName(GithubCredentialsDialog)
    # setupUi

    def retranslateUi(self, GithubCredentialsDialog):
        GithubCredentialsDialog.setWindowTitle(QCoreApplication.translate("GithubCredentialsDialog", u"Dialog", None))
        self.groupBox.setTitle(QCoreApplication.translate("GithubCredentialsDialog", u"Github credentials", None))
        self.accessToken_radio.setText(QCoreApplication.translate("GithubCredentialsDialog", u"Access token", None))
        self.userpass_radio.setText(QCoreApplication.translate("GithubCredentialsDialog", u"User/pass", None))
        self.label_3.setText(QCoreApplication.translate("GithubCredentialsDialog", u"Access token:", None))
        self.label.setText(QCoreApplication.translate("GithubCredentialsDialog", u"User name:", None))
        self.label_2.setText(QCoreApplication.translate("GithubCredentialsDialog", u"Password:", None))
        self.label_4.setText(QCoreApplication.translate("GithubCredentialsDialog", u"New branch name:", None))
        self.label_5.setText(QCoreApplication.translate("GithubCredentialsDialog", u"Local repo folder:", None))
        self.localFolder_Browse_Button.setText(QCoreApplication.translate("GithubCredentialsDialog", u"Browse...", None))
    # retranslateUi

